﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;
using System.Web;

namespace X03
{
	public class Page : IDisposable
	{
		private string _path;

		// ページが含んでいるすべての要素、ネストしている分も含む
		public static Dictionary<string, PageElement> Elements { get; set; }

		public string Wikiname { get; set; }
		public string Wikitext { get; set; }
		public PageElements.Document Document { get; set; }
		public string Html { get; set; }

		/// <summary>
		/// static constructor
		/// </summary>
		static Page()
		{
			Page.Elements = new Dictionary<string, PageElement>();
		}

		//public ScoreBook SimilarityTo(Page that, ScoreBook scoreBook_Query)
		//{
		//    return this.Document.SimilarityTo(that.Document);
		//}

		static public string PageElementToString(PageElement element)
		{
			string ret = "";
			foreach (var el in element.Children)
			{
				if (el.Children != null && el.Children.Count >= 1)
				{
					ret += "[" + el.GetType().Name + "]<br />" + @"<div style = ""padding: 0 0 0 4em;"">" + PageElementToString(el) + "</div>";
				}
				else
				{
					ret += "[" + el.GetType().Name + @"]:""" + Regex.Replace(el.Wikitext, @"(?:\r\n|\r|\n)", @"\n") + @"""<br />" + "\n";
				}
			}
			return ret;
		}

		public Page()
		{
			this.Document = new PageElements.Document(this, null);
			Page.Elements.Add(this.GetHashCode().ToString(), this.Document);
		}

		public Page(string name, string content)
		{
			this.Wikiname = name;
			this._path = @"~/App_Data/" + name + ".wiki.txt";

			this.Wikitext = content;
			//contentを与えられているのでLoad()しない
			//this.Load();

			var ignoreClasses = new HashSet<Type>();
			this.Document = new PageElements.Document(this, null);
			List<PageElement> tokens = PageElements.Notation.Tokenize(this.Document, System.Web.HttpUtility.HtmlEncode(this.Wikitext), ignoreClasses);
			this.Document.Children = tokens;

			// PageElement -> HTML
			this.Document.InAction();
			this.Html = (string)this.Document.ToHtml();
		}

		public Page(string name)
		{
			this.Wikiname = name;
			this._path = @"~/App_Data/" + name + ".wiki.txt";

			this.Load();

			var ignoreClasses = new HashSet<Type>();
			// Document:全Element or null; Children:Documentの子
			this.Document = new PageElements.Document(this, null);
			List<PageElement> tokens = PageElements.Notation.Tokenize(this.Document, System.Web.HttpUtility.HtmlEncode(this.Wikitext), ignoreClasses);
			this.Document.Children = tokens;

			// PageElement -> HTML
			this.Document.InAction();
			this.Html = (string)this.Document.ToHtml();
		}

		public void Dispose()
		{
			this.Save();
		}

		public void Load()
		{
			System.Web.HttpServerUtility server = HttpContext.Current.Server;
			using (StreamReader p = new StreamReader(server.MapPath(this._path)))
			{
				this.Wikitext = p.ReadToEnd();
			}
		}

		public void Save()
		{
			System.Web.HttpServerUtility server = HttpContext.Current.Server;
			using (StreamWriter p = new StreamWriter(server.MapPath(this._path)))
			{
				p.Write(this.Wikitext);
			}
		}
	}
}
