﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Reflection;

namespace X03
{
	public class ScoreBook
	{
		public Dictionary<string, float> Score { get; set; }
		public Dictionary<string, Dictionary<string, int>> Distance { get; set; }

		public ScoreBook()
		{
			this.Score = new Dictionary<string, float>();
			this.Distance = new Dictionary<string, Dictionary<string, int>>();
		}
	}

	//public struct ListListNode
	//{
	//    public string String;
	//    public List<List<ListListNode>> Node;
	//}

	/// <summary>
	/// ページの要素。
	/// リフレクションを使用するクラス。
	/// </summary>
	abstract public class PageElement : Plugin.PageElement, PageElementAcceptor
	{
		public PageElement Parent { get; set; }
		public List<PageElement> Children { get; set; }
		public string WikitextPre { get; protected set; }
		public string WikitextPost { get; protected set; }
		public string Wikitext
		{
			get
			{
				return this.WikitextPre + this.WikitextPost;
			}
		}

		abstract public void Accept(PageElementVisitor visitor);

		//public void Accept(PageElementVisitor visitor)
		//{
		//    visitor.Visit(this);
		//}

		public List<PageElement> Ancestors
		{
			get
			{
				var ret = new List<PageElement>();
				if (this.Parent != null)
				{
					ret.Add(this.Parent);
					ret.AddRange(this.Parent.Ancestors);
				}
				Debug.Assert(this.Parent == null ? this is PageElements.Document : true);
				return ret;
			}
		}

		public object ConvertTo(string dataType)
		{
			MethodInfo m = this.GetType().GetMethod("To" + dataType);
			if (m == null)
				return null;
			else
				return m.Invoke(this, new object[] { });
		}

		public List<PageElement> GetAllDescendants()
		{
			var ret = new List<PageElement>();

			ret.AddRange(this.Children);
			foreach (var elem in this.Children)
			{
				ret.AddRange(elem.GetAllDescendants());
			}

			return ret;
		}

		//public void CallChildAction(PageElement child)
		//{
		//    child.GetType().GetMethod("InAction").Invoke(child, null);
		//}

		abstract public void InAction();

		public object[] RoundRobinDescendants(PageElement that, Delegate visitor)
		{
			System.Diagnostics.Debug.WriteLine(this.GetType().Name.ToString() + ".RoundRobinDescendants() Start");
			var ret = new List<object>();

			//FIXME:同じobjの組み合わせが重複している
			if (this.Children != null)
				foreach (var thisElement in this.Children)
				{
					// that側を引数に
					ret.AddRange(thisElement.RoundRobinDescendants(that, visitor));
				}
			if (that.Children != null)
				foreach (var thatElement in that.Children)
				{
					// that側を引数に
					ret.AddRange(this.RoundRobinDescendants(thatElement, visitor));
				}

			{
				// call
				ret.Add(visitor.Method.Invoke(null, new object[] { this, that }));
				//Debug.Assert(r.ContainsKey("query") && r.ContainsKey("target"));
				//foreach (var k in ((Dictionary<string, float>)r).Keys)
				//    Debug.WriteLine(((Dictionary<string, float>)r)[k].ToString() + @": """ + this.WikiText + @""" : """ + that.WikiText + @"""");
			}

			System.Diagnostics.Debug.WriteLine(this.GetType().Name.ToString() + "(" + this.GetHashCode().ToString() + ").RoundRobinDescendants(" + that.GetHashCode().ToString() + ") End");
			return ret.ToArray();
		}

		abstract public string CreateTag();

		public override string ToString()
		{
			return this.GetType().FullName.ToString() + ":" + this.Wikitext;
		}

		// 幅優先再帰呼び出しでChildrenを線形リスト化
		public PageElement[] LinerElements()
		{
			var list = new List<PageElement>();

			list.AddRange(this.Children);

			var listEnum = (IEnumerator<PageElement>)list.GetEnumerator();
			while (listEnum.MoveNext())
			{
				var listEnum2 = (IEnumerator<PageElement>)listEnum.Current.Children.GetEnumerator();
				while (listEnum2.MoveNext())
				{
					list.AddRange(listEnum2.Current.LinerElements());
				}
			}

			return list.ToArray();
		}

		public PageElement(PageElement parent)
			: this()
		{
			this.Parent = parent;
		}

		public PageElement()
		{
			this.WikitextPre = "";
			this.WikitextPost = "";
			this.Children = new List<PageElement>();
			Page.Elements[this.GetHashCode().ToString()] = this;
		}
	}
}
