﻿using System.Diagnostics;
using System.Text;

namespace X03.PageElements
{
	public class Document : PageElement, Plugin.PageElement, Plugin.Findable
	{
		override public void Accept(PageElementVisitor visitor)
		{
			visitor.Visit(this);
		}

		public Page ParentPage { get; set; }

		public string ToWikitext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("WikiText");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		public string ToHtml()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Html");
				if (cret != null)
					ret.Append((string)cret);
			}

			return @"<div class=""document"">" + "\n" + ret + "</div>\n";	// this.wikiText不要
		}

		public string ToPlaintext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Plaintext");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		public override void InAction()
		{
			foreach (var child in this.Children)
				child.InAction();
		}

		override public string CreateTag()
		{
			return this.GetType().Name + ":" + "";
		}

		public float SimilarityTo2(PageElement that)
		{
			return 1F;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="parentPage"></param>
		/// <param name="parent">null 使用されるnull</param>
		public Document(Page parentPage, PageElement parent)
			: this(parent)
		{
			Debug.Assert(parent == null);

			this.ParentPage = parentPage;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="parent">null 使用されるnull</param>
		public Document(PageElement parent)
			: base(parent)
		{
			Debug.Assert(parent == null);

			this.WikitextPre = "";
			this.WikitextPost = "";
		}
	}
}
