﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace X03.PageElements.Notations
{
	public class Date : PageElements.Notation, Plugin.PageElement, Plugin.Findable
	{
		override public void Accept(PageElementVisitor visitor)
		{
			visitor.Visit(this);
		}

		public string ToWikitext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("WikiText");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		public string ToHtml()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Html");
				if (cret != null)
					ret.Append((string)cret);
			}

			return "《" + this.Value + ret + "》";
		}

		public string ToPlaintext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Plaintext");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		public override void InAction()
		{
			foreach (var child in this.Children)
				child.InAction();
		}

		public DateTime Value { get; set; }

		override public string CreateTag()
		{
			return this.GetType().Name + ":" + "";
		}

		public float SimilarityTo2(PageElement that)
		{
			TimeSpan diff = this.Value - ((PageElements.Notations.Date)that).Value;
			if (diff < TimeSpan.Zero)
				diff = diff.Negate();
			float score = (float)(1D / (Math.Floor((double)diff.Ticks / TimeSpan.TicksPerDay) + 1));	// score:(差(日数) + 1)の逆数

			return score;
		}

		new static public string Pattern
		{
			get { return @"(?<!\d)((?:\d\d\d\d/\d\d/\d\d|\d\d/\d\d|\d\d\d\d/\d\d))(\s*\((?:Mon|Tue|Wed|Thu|Fri|Sat|Sun)\))?(?!\d)"; }
		}

		public Date(PageElement parent, Match m, HashSet<Type> ignoreClasses)
			: base(parent, m, ignoreClasses)
		{
			this.WikitextPre = m.Groups[0].Value;
			this.WikitextPost = "";
			string content = m.Groups[1].Value;
			string content2 = m.Groups[2].Value;
			this.Value = DateTime.Parse(content);

			//Dateではこれ以上解析しない。
			//elems.AddRange(Notation.Tokenize(Regex.Replace(content, @"/", "-"), ignoreClasses));
		}
	}
}
