﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;

namespace X03.PageElements.Notations
{
	public class Find : PageElements.Notation, Plugin.PageElement, Plugin.Findable
	{
		private const int _PreviewLength = 3;

		override public void Accept(PageElementVisitor visitor)
		{
			visitor.Visit(this);
		}

		public string QueryString { get; set; }
		private List<PageElement> _QueryElements;
		private List<PageElement> _PageSetElements;
		private Dictionary<string, List<string>> _Result;
		private Dictionary<string, float> _Ranking;
		private Dictionary<string, List<string>> _EachResultOfQuery;

		public Dictionary<string, List<string>> ToDictionaryList(PageElement parent)
		{
			return this._Result;
		}

		public List<Dictionary<string, string>> ToListDictionary(PageElement parent)
		{
			var ret = new List<Dictionary<string, string>>();

			foreach (var r in this._Result)
			{
				var d = new Dictionary<string, string>();
				d.Add("page", r.Value[0]);
				d.Add("score", r.Value[1]);
				ret.Add(d);
			}

			return ret;
		}

		public string ToWikitext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("WikiText");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		public string ToHtml()
		{
			var ret = new StringBuilder();

			//FIXME:（クラスごとではなく）objごとのクエリー文字列入力に
			if (this._QueryElements.Count == 0 || this.QueryString != null)
			{
				// 検索フォーム
				ret.AppendLine(@"<form method=""get"" action=""" + Global.Request.Url.AbsoluteUri + @""" style=""margin:1ex;"">");
				ret.AppendLine(@"query: <input type=""text"" name=""" + this.GetType().FullName.ToString() + @""" value=""" + this.QueryString + @""" style=""padding:0.5ex;"" />");
				ret.AppendLine(@"<input type=""submit"" style=""padding:0.5ex 1ex;"" /></form>");
			}
			else
			{
				ret.Append(@"<div style=""margin:1ex;"">query: ");
				ret.Append(@"<span style=""border:thin black solid; padding:0.5ex 1ex;"">");
				foreach (var elem in this._QueryElements)
				{
					ret.Append((string)elem.ConvertTo("Wikitext"));
				}
				ret.Append("</span></div>");
				ret.AppendLine();
			}

			if (this._QueryElements.Count != 0)
			{
				ret.AppendLine("<table>");
				ret.Append("<tr>");
				ret.Append("<th>rank</th>");
				ret.Append("<th>page</th>");
				ret.Append("<th>page content</th>");

				//foreach (var qe in this._QueryElements)
				//{
				//    ret.Append(@"<th>base score<br /> query:""" + qe.Wikitext + @"""</th>");
				//}

				ret.Append(@"<th style=""border-color:#a00; background-color:#a44; color:white;"">Page preview</th>");
				ret.Append(@"<th style=""border-color:#a00; background-color:#a44; color:white;"">Page score</th>");
				foreach (var qe in this._QueryElements)
				{
					ret.Append(@"<th style=""border-color:#080; background-color:#484; color:white;"">result for query: """ + qe.Wikitext + @"""<br /><button type=""button"" disabled=""disabled"">ここにソートするボタンを置く</button></th>");
				}
				var numcolumn = this._Result.Where(p => p.Value.Count == this._Result.Max(pp => pp.Value.Count)).First().Value.Count;
				for (var idx = 0; idx < numcolumn - 2 - this._QueryElements.Count - 2; ++idx) // -2:pagename,content; -2:preview,previewscore
				{
					ret.Append("<th>page<br />flagment" + (idx + 1).ToString() + "</th>");
				}
				ret.Append("</tr>");
				ret.AppendLine();

				var rank = 1;
				foreach (var p in this._Ranking.OrderByDescending(p => p.Value))
				{
					var value = this._Result[p.Key];
					ret.Append("<tr>");
					ret.Append("<td>" + rank.ToString() + "</td>");
					ret.Append("<td>" + value[0] + "</td>");
					ret.Append(@"<td style=""white-space:pre;"">" + value[1] + "</td>");

					//foreach (var sc in value.Skip(2).Take(this._QueryElements.Count))
					//{
					//    ret.Append("<td>" + sc + "pts.</td>");
					//}

					// preview
					ret.Append(@"<td  style=""border-color:#a00; white-space:pre;"">" + value[2 + this._QueryElements.Count + 1 - 1] + "</td>");
					// score
					ret.Append(@"<td  style=""border-color:#a00;"">" + value[2 + this._QueryElements.Count + 2 - 1] + "</td>");

					// each score of query
					foreach (var rq in this._EachResultOfQuery[p.Key])
					{
						ret.Append(@"<td  style=""border-color:#080;"">" + rq + "</td>");
					}

					// fragment
					foreach (var sc in value.Skip(2 + this._QueryElements.Count + 3 - 1))
					{
						ret.Append("<td>" + sc + "</td>");
					}
					ret.Append("</tr>");
					ret.AppendLine();

					rank++;
				}

				ret.AppendLine("</table>");
			}

			return ret.ToString();
		}

		public string ToPlaintext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Plaintext");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		// 戻り値がobjectなのはDelegateの仕様による
		//public static Func<PageElement, PageElement, object> similarityTo = delegate(PageElement query, PageElement target)
		public static Func<PageElement, PageElement, object> similarityTo = delegate(PageElement query, PageElement target)
		{
			var ret = new Dictionary<string, Dictionary<string, float>>();
			var scoreBook = new Dictionary<string, float>();
			var scoreBook2 = new Dictionary<string, float>();
			var targetToQuery = new Dictionary<string, string>();

			if (query.GetType() == target.GetType())
			{
				if (query is Plugin.Findable)
				{
					MethodInfo similarityTo2 = query.GetType().GetMethod("SimilarityTo2");
					Debug.Assert(similarityTo2 != null);
					var similarity = (float)similarityTo2.Invoke(query, new object[] { target });
					// クエリーごとにスコア算出するので、scoreBookのキーはqueryから求める
					if (!scoreBook.ContainsKey(query.GetHashCode().ToString()) || scoreBook[query.GetHashCode().ToString()] < similarity)
						scoreBook[query.GetHashCode().ToString()] = similarity;
					if (!scoreBook2.ContainsKey(target.GetHashCode().ToString()) || scoreBook2[target.GetHashCode().ToString()] < similarity)
						scoreBook2[target.GetHashCode().ToString()] = similarity;
					targetToQuery[target.GetHashCode().ToString()] = query.GetHashCode().ToString();

					//var score = (float)similarityTo2.Invoke(this, new object[] { that, parent });
					//if (!scoreBook_Query.ContainsKey(this.GetType().Name) || scoreBook_Query[this.GetType().Name] < score)
					//{
					//    scoreBook_Query[this.GetType().Name] = score;
					//}
				}
			}

			//return scoreBook_Query;
			return new Dictionary<string, object>() { { "query", scoreBook }, { "target", scoreBook2 }, { "targetToQuery", targetToQuery } };
		};

		public override void InAction()
		{
			foreach (var child in this.Children)
				child.InAction();
			foreach (var child in this._PageSetElements)
				child.InAction();

			// 類似度を使った検索
			if (this._QueryElements.Count != 0)
			{

				this._Result.Clear();
				this._Ranking.Clear();
				foreach (var targetProvider in this._PageSetElements)
				{
					var tret = targetProvider.ConvertTo("PageSet");
					if (tret != null)
					{
						var pagenames = (string[])tret;
						foreach (var pagename in pagenames)
						{
							var scoreBook_Query = new Dictionary<string, float>();
							var scoreBook_Target = new Dictionary<string, float>();
							var targetToQuery = new Dictionary<string, string>();
							var guid = Guid.NewGuid();

							//TODO:類似度比較なのでthisが存在するページの名前取得、同名のページは処理対象外。
							//if (name == "Default") continue;
							var thisDoc = (PageElements.Document)this.Ancestors.Last();
							Debug.Assert(thisDoc is PageElements.Document);
							if (pagename == thisDoc.ParentPage.Wikiname) continue;

							Page target = PageFactory.getInstance().getPage(pagename);
							{
								foreach (var qe in this._QueryElements)
								{

									//Debug.WriteLine("RoundRobinDescendants ret:" + rets.Length.ToString());

									// 検索結果の集約
									foreach (Dictionary<string, object> r in qe.RoundRobinDescendants(target.Document, similarityTo))
									{
										foreach (var p in (Dictionary<string, float>)r["query"])
										{
											if (scoreBook_Query.ContainsKey(p.Key))
											{
												if (scoreBook_Query[p.Key] < p.Value)
													scoreBook_Query[p.Key] = p.Value;
											}
											else
											{
												scoreBook_Query.Add(p.Key, p.Value);
											}
										}
										foreach (var p in (Dictionary<string, float>)r["target"])
										{
											if (scoreBook_Target.ContainsKey(p.Key))
											{
												if (scoreBook_Target[p.Key] < p.Value)
													scoreBook_Target[p.Key] = p.Value;
											}
											else
											{
												scoreBook_Target.Add(p.Key, p.Value);
											}
										}
										foreach (var p in (Dictionary<string, string>)r["targetToQuery"])
										{
											targetToQuery[p.Key] = p.Value;
										}
									}
								}
							}
							//scoreBook_Query = this._QueryPage.SimilarityTo(target2, scoreBook_Query);

							//scoreBookに記録されているElementID全てについて、ElementID→Element、他の全Elementとの距離算出、ElementID→スコア += (相手Elementスコア / 距離) 。近いElement同士でスコアを高める。
							var distance = new Dictionary<string, float>();
							foreach (var k in scoreBook_Target.Keys)
							{
								foreach (var k2 in scoreBook_Target.Keys)
								{
									// 共通の祖先を発見
									// その子(.Elements)-kの祖先との共通点、その子(.Elements)-k2の祖先との共通点
									// …の（.Elements上での）距離の逆数→kの近さ値に加算
									var t = Page.Elements[k];
									var t2 = Page.Elements[k2];
									var ancestor = t.Ancestors.Intersect(t2.Ancestors).Last();
									Debug.Assert(ancestor is Document ? t.Ancestors.Intersect(t2.Ancestors).Count() == 1 : t.Ancestors.Intersect(t2.Ancestors).Count() > 1);

									// 検索にヒットしたページ側要素同士の距離を加味
									var idxq = ancestor.Children.IndexOf(t);
									var idxt = ancestor.Children.IndexOf(t2);
									if (!distance.ContainsKey(t2.GetHashCode().ToString()))
										distance[t2.GetHashCode().ToString()] = 0;
									var d = (Math.Abs(idxq - idxt) == 0) ? scoreBook_Target[k] : scoreBook_Target[k] / Math.Abs(idxq - idxt);	// 0.5Fでバランス調整
									if (ancestor.Children.Count - d < d)
									{
										// ancestor.Childrenの最初と最後をつないで環を作ったとして、外回りの距離のほうが近ければそちらを使用
										d = ancestor.Children.Count - d;
									}
									distance[t2.GetHashCode().ToString()] += d;
								}
							}
							// 距離を反映
							foreach (var p in distance)
							{
								scoreBook_Target[p.Key] *= p.Value;
							}

							// scoreBook_Targetに載っているobj（ページ側）を、ヒットした相手（クエリー側obj）別に分類。this._Resultに追加
							//TODO:ソートキーになっている分類でソートされるように
							var eachTopTargetOfQuery = new Dictionary<string, string>();
							{
								var eachTargetOfQuery = new Dictionary<string, List<string>>();
								foreach (var tp in scoreBook_Target)
								{
									var qk = targetToQuery[tp.Key];
									if (!eachTargetOfQuery.ContainsKey(qk))
										eachTargetOfQuery[qk] = new List<string>();
									eachTargetOfQuery[qk].Add(tp.Key);
								}
								foreach (var qp in scoreBook_Query)
								{
									eachTargetOfQuery[qp.Key].ForEach(tk => Debug.Assert(scoreBook_Target.ContainsKey(tk)));
									eachTopTargetOfQuery[qp.Key] = eachTargetOfQuery[qp.Key].Where(tk => scoreBook_Target[tk] == eachTargetOfQuery[qp.Key].Max(tk2 => scoreBook_Target[tk2])).First();
								}
							}

							{
								var l = new List<string>();
								l.Add(target.Wikiname);
								l.Add(target.Wikitext);
								foreach (var sc in scoreBook_Query.Values)
									l.Add(sc.ToString());	// FIXME:使ってない

								if (scoreBook_Target.Count > 0)
								{
									// Preview
									var v = new PageElementVisitors.PageElementCollector();
									target.Document.Accept(v);
									var k = scoreBook_Target.Where(p => p.Value == scoreBook_Target.Values.Max()).First();
									var top = Page.Elements[k.Key];
									var idx = v.Collection.IndexOf(new PageElementVisitors.PageElementCollector.Tuple(top, "pre"));
									var str = new StringBuilder();

									Debug.Assert(idx <= v.Collection.Count - 1);

									for (var i = Math.Max(idx - _PreviewLength * 2, 0); 0 <= i && i <= idx - 1; ++i)
									{
										var e = v.Collection.ElementAt(i);

										if (e.Position == "pre")
											str.Append(e.Element.WikitextPre);
										else if (e.Position == "post")
											str.Append(e.Element.WikitextPost);
										else
											Debug.Fail("");
									}
									{
										str.Append("<strong>" + v.Collection.ElementAt(idx).Element.Wikitext + "</strong>");
									}
									for (var i = idx + 1; i <= idx + (_PreviewLength * 2 - 1) && i <= v.Collection.Count - 1; ++i)
									{
										var e = v.Collection.ElementAt(i);

										if (e.Position == "pre")
											str.Append(e.Element.WikitextPre);
										else if (e.Position == "post")
											str.Append(e.Element.WikitextPost);
										else
											Debug.Fail("");
									}

									l.Add(str.ToString());

									// page score
									l.Add(k.Value.ToString() + "pts.");
									_Ranking.Add(guid.ToString(), k.Value);

									// each score of query
									foreach (var qe in this._QueryElements)
									{
										var tkey = eachTopTargetOfQuery[qe.GetHashCode().ToString()];
										if (!this._EachResultOfQuery.ContainsKey(guid.ToString()))
											this._EachResultOfQuery[guid.ToString()] = new List<string>();
										this._EachResultOfQuery[guid.ToString()].Add("<strong>" + Page.Elements[tkey].Wikitext + "</strong><br />" + scoreBook_Target[tkey].ToString() + "pts.");
									}
								}
								else
								{
									l.Add("0");	// FIXME:使ってない

									// Preview
									l.Add("");

									// page score
									l.Add("0");
									_Ranking.Add(guid.ToString(), 0);

									// each score of query
									this._EachResultOfQuery[guid.ToString()] = new List<string>();
									this._EachResultOfQuery[guid.ToString()].Add("0pts.");
								}

								foreach (var p2 in scoreBook_Target)
								{
									l.Add("<strong>" + Page.Elements[p2.Key].Wikitext + "</strong><br />" + p2.Value.ToString() + "pts.");
								}
								this._Result.Add(guid.ToString(), l);

								Debug.Assert(this._Result.Count == this._Ranking.Count);
							}
						}
					}
				}
			}
		}

		public override string CreateTag()
		{
			return this.GetType().Name + ":" + "";
		}

		public float SimilarityTo2(PageElement that)
		{
			return 1F;
		}

		static public new string Pattern
		{
			get { return @"&amp;find\(([^\),]*)(,)([^\),]*)\);"; }
		}

		public Find(PageElement parent, Match m, HashSet<Type> ignoreClasses)
			: base(parent, m, ignoreClasses)
		{
			this.WikitextPre = m.Groups[0].Value;
			this.WikitextPost = "";
			string content = m.Groups[1].Value;
			string delimiter = m.Groups[2].Value;
			string content2 = m.Groups[3].Value;

			this._QueryElements = new List<PageElement>();
			this._PageSetElements = new List<PageElement>();
			//this._QueryPage = null;
			this._Result = new Dictionary<string, List<string>>();
			this._Ranking = new Dictionary<string, float>();
			this._EachResultOfQuery = new Dictionary<string, List<string>>();

			{
				ignoreClasses.Add(this.GetType());

				if (content != "")
				{
					var elems = PageElements.Notation.Tokenize(parent, content, ignoreClasses);
					this._QueryElements.AddRange(elems);
					this.Children.AddRange(elems);
				}
				if (delimiter != "")
				{
					this.Children.AddRange(PageElements.Notation.Tokenize(parent, delimiter, ignoreClasses));
				}
				if (content2 != "")
				{
					var elems = PageElements.Notation.Tokenize(parent, content2, ignoreClasses);
					this._PageSetElements.AddRange(elems);
					this.Children.AddRange(elems);
				}

				ignoreClasses.Remove(this.GetType());
			}

			if (this._QueryElements.Count == 0)
			{
				this.QueryString = HttpUtility.HtmlEncode(Global.Request.QueryString[this.GetType().FullName.ToString()]);

				Debug.Assert(this.QueryString == null || this.QueryString != null);

				if (this.QueryString != null)
				{
					ignoreClasses.Add(this.GetType());
					this._QueryElements = PageElements.Notation.Tokenize(parent, this.QueryString, ignoreClasses);
					ignoreClasses.Remove(this.GetType());
				}
			}

		}
	}
}
