﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace X03.PageElements.Notations
{
	public class Include : PageElements.Notation, Plugin.PageElement, Plugin.Findable
	{
		override public void Accept(PageElementVisitor visitor)
		{
			visitor.Visit(this);
		}

		public string ToWikitext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("WikiText");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.WikitextPre + ret + this.WikitextPost;
		}

		public string ToHtml()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Html");
				if (cret != null)
					ret.Append((string)cret);
			}

			return @"(include:""" + ret + @""")";	// this.wikiText不要
		}

		public string ToPlaintext()
		{
			throw new NotImplementedException();
		}

		public override void InAction()
		{
			foreach (var child in this.Children)
				child.InAction();
		}

		override public string CreateTag()
		{
			return this.GetType().Name + ":" + "";
		}

		public float SimilarityTo2(PageElement that)
		{
			return 1F;
		}

		new static public string Pattern
		{
			// @"\{(.*?)\}" でないのはネストを可能にするため
			get { return @"(\{)([^\{\}]*?)(\})"; }
		}

		public Include(PageElement parent, Match m, HashSet<Type> ignoreClasses)
			: base(parent, m, ignoreClasses)
		{
			this.WikitextPre = m.Groups[1].Value;
			string content = m.Groups[2].Value;
			this.WikitextPost = m.Groups[3].Value;

			ignoreClasses.Add(this.GetType());
			this.Children.AddRange(PageElements.Notation.Tokenize(parent, content, ignoreClasses));
			ignoreClasses.Remove(this.GetType());
		}
	}
}
