﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace X03.PageElements.Notations
{
	public class Link : PageElements.Notation, Plugin.PageElement, Plugin.Findable
	{
		override public void Accept(PageElementVisitor visitor)
		{
			visitor.Visit(this);
		}

		public string ToWikitext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("WikiText");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		public string ToHtml()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Html");
				if (cret != null)
					ret.Append((string)cret);
			}

			return @"<a href=""about:blank"">" + ret + "</a>";	// this.wikiText不要
		}

		public string ToPlaintext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Plaintext");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		public override void InAction()
		{
			foreach (var child in this.Children)
				child.InAction();
		}

		public override string CreateTag()
		{
			return this.GetType().Name + ":" + "";
		}

		public float SimilarityTo2(PageElement that)
		{
			return 1F;
		}

		static public new string Pattern
		{
			//			get { return @"\[\[(.*?)\]\]"; }
			get { return @"(\[\[)([^\[\]]*?)(\]\])"; }
		}

		public Link(PageElement parent, Match m, HashSet<Type> ignoreClasses)
			: base(parent, m, ignoreClasses)
		{
			this.WikitextPre = m.Groups[0].Value;
			string content = m.Groups[1].Value;
			this.WikitextPost = m.Groups[2].Value;

			ignoreClasses.Add(this.GetType());
			this.Children.AddRange(PageElements.Notation.Tokenize(parent, content, ignoreClasses));
			ignoreClasses.Remove(this.GetType());
		}

	}
}
