﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace X03.PageElements.Notations
{
	public class Li : PageElements.Notation, Plugin.PageElement, Plugin.Findable
	{
		override public void Accept(PageElementVisitor visitor)
		{
			visitor.Visit(this);
		}

		public string ToWikitext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("WikiText");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		public string ToHtml()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Html");
				if (cret != null)
					ret.Append((string)cret);
			}

			return "<li>" + ret + "</li>\n"; // this.wikiText不要
		}

		public string ToPlaintext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Plaintext");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		public override void InAction()
		{
			foreach (var child in this.Children)
				child.InAction();
		}

		override public string CreateTag()
		{
			return this.GetType().Name + ":" + "";
		}

		public float SimilarityTo2(PageElement that)
		{
			return 1F;
		}

		new static public string Pattern
		{
			// Tokenize()に参加しないためのnull。
			get { return null; }
			//get { return @"^\+(.+)"; }
			//get { return @"^(.+)"; }
		}

		public Li(PageElement parent, Match m, HashSet<Type> ignoreClasses)
			: base(parent, m, ignoreClasses)
		{
			this.WikitextPre = m.Groups[0].Value;
			string content = m.Groups[0].Value;
			this.WikitextPost = "";

			ignoreClasses.Add(this.GetType());
			this.Children.AddRange(PageElements.Notation.Tokenize(parent, content, ignoreClasses));
			ignoreClasses.Remove(this.GetType());
		}
	}

	public class Ol : PageElements.Notation, Plugin.PageElement
	{
		override public void Accept(PageElementVisitor visitor)
		{
			visitor.Visit(this);
		}

		public string ToWikitext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("WikiText");
				if (cret != null)
					ret.Append((string)cret);
			}

			return ret.ToString();
		}

		public string ToHtml()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Html");
				if (cret != null)
					ret.Append((string)cret);
			}

			return "<ol>\n" + ret + "</ol>\n";	// this.wikiText不要
		}

		public string ToPlaintext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Plaintext");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		public override void InAction()
		{
			foreach (var child in this.Children)
				child.InAction();
		}

		override public string CreateTag()
		{
			return this.GetType().Name + ":" + "";
		}

		public float SimilarityTo2(PageElement that, PageElement parent)
		{
			return 1F;
		}

		new static public string Pattern
		{
			get { return @"(?<=\r\n|\r|\n|^)\+(.+?)(?:\r\n|\r|\n|$)"; }
		}

		/// <summary>
		/// 連結
		/// </summary>
		/// <param name="m"></param>
		/// <param name="ignoreClasses"></param>
		/// <param name="target"></param>
		static public void Merge(PageElement parent, Match m, HashSet<Type> ignoreClasses, PageElements.Notations.Ol target)
		{
			ignoreClasses.Add(target.GetType());
			target.Children.Add(new PageElements.Notations.Li(parent, m, ignoreClasses));
			ignoreClasses.Remove(target.GetType());
		}

		public Ol(PageElement parent, Match m, HashSet<Type> ignoreClasses)
			: base(parent, m, ignoreClasses)
		{
			ignoreClasses.Add(this.GetType());
			this.Children.Add(new PageElements.Notations.Li(parent, m, ignoreClasses));
			ignoreClasses.Remove(this.GetType());
			// LiコンストラクターでTokenize()
		}

	}
}
