﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace X03.PageElements.Notations
{
	public class Table : PageElements.Notation, Plugin.PageElement, Plugin.Findable
	{
		override public void Accept(PageElementVisitor visitor)
		{
			visitor.Visit(this);
		}

		public List<List<PageElement>> ListList;
		protected string Content { get; set; }

		public string ToWikitext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("WikiText");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		public string ToHtml()
		{
			var ret = new StringBuilder();

			ret.Append("\n<table>\n");

			foreach (var rows in this.ListList)
			{
				ret.Append("\n<tr>\n");
				foreach (var child in rows)
				{
					var cret = child.ConvertTo("Html");
					if (cret != null)
					{
						ret.Append("<td>" + (string)cret + "</td>");
					}
				}
				ret.Append("\n</tr>\n");
			}

			ret.Append("\n</table>\n");

			return ret.ToString();
		}

		public string ToPlaintext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Plaintext");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		public override void InAction()
		{
			foreach (var child in this.Children)
				child.InAction();
		}

		override public string CreateTag()
		{
			return this.GetType().Name + ":" + "";
		}

		public float SimilarityTo2(PageElement that)
		{
			return 1F;
		}

		new static public string Pattern
		{
			get { return @"(?<=\r\n|\r|\n|^)((?:\|[^\r\n]*?)+\|(?:\r\n|\r|\n|$))"; }
		}

		/// <summary>
		/// 連結
		/// WikiTextの保存だけ。
		/// </summary>
		/// <param name="parent"></param>
		/// <param name="m"></param>
		/// <param name="ignoreClasses"></param>
		/// <param name="target"></param>
		static public void Merge(PageElement parent, Match m, HashSet<Type> ignoreClasses, PageElements.Notations.Table target)
		{
			target.WikitextPre += m.Groups[0].Value;
			target.Content += m.Groups[1].Value;
			var t = new PageElements.Notations.Table(parent, m, ignoreClasses);
			target.ListList.Add(t.Children);
			target.Children.AddRange(t.Children);
		}

		public Table(PageElement parent, Match m, HashSet<Type> ignoreClasses)
			: base(parent, m, ignoreClasses)
		{
			this.ListList = new List<List<PageElement>>();
			this.WikitextPre = "";
			this.Content = "";

			if (m != null)
			{
				this.WikitextPre = m.Groups[0].Value;
				this.Content = m.Groups[1].Value;

				// セルごとに要素化
				var elems = new List<PageElement>();
				foreach (var t in Regex.Split(Regex.Replace(this.Content.Trim(), @"^\||\|$", "", RegexOptions.Singleline), @"\|"))
				{
					ignoreClasses.Add(this.GetType());
					elems.AddRange(PageElements.Notation.Tokenize(parent, t, ignoreClasses));
					ignoreClasses.Remove(this.GetType());
				}
				if (elems != null)
				{
					this.Children.AddRange(elems);
					this.ListList.Add(elems);	// 二重構造リスト
				}
			}
		}
	}
}
