﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Text.RegularExpressions;

namespace X03.PageElements.Plains
{
	public class Plaintext : PageElements.Plain, Plugin.PageElement, Plugin.Findable
	{

		override public void Accept(PageElementVisitor visitor)
		{
			visitor.Visit(this);
		}

		public string ToWikitext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("WikiText");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		public string ToHtml()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Html");
				if (cret != null)
					ret.Append((string)cret);
			}

			var html = Regex.Replace(this.Wikitext, @"\n", "<br />\n", RegexOptions.Singleline);	// \n -> <br />
			return html + ret;
		}

		public string ToPlaintext()
		{
			var ret = new StringBuilder();

			foreach (var child in this.Children)
			{
				var cret = child.ConvertTo("Plaintext");
				if (cret != null)
					ret.Append((string)cret);
			}

			return this.Wikitext + ret;
		}

		public override void InAction()
		{
			foreach (var child in this.Children)
				child.InAction();
		}

		public override string CreateTag()
		{
			return this.GetType().Name + ":" + "";
		}

		public float SimilarityTo2(PageElement that)
		{
			float score;


			string thisText;
			{
				var v = new PageElementVisitors.WikitextCollector();
				this.Accept(v);
				thisText = v.Collection;
			}
			string thatText;
			{
				var v = new PageElementVisitors.WikitextCollector();
				((PageElements.Plains.Plaintext)that).Accept(v);
				thatText = v.Collection;
			}

			if (thisText.Length < 2)
			{
				// bi-gram使用不可能な場合
				score = (thatText.IndexOf(thisText) != -1) ? 1F : 0F;
			}
			else
			{
				// bi-gram
				var thatd = new Dictionary<string, int>();
				string s_ = null;
				foreach (var s in Regex.Split(thatText, @"", RegexOptions.Singleline))
				{
					if ((s ?? "") != "" && (s_ ?? "") != "")
					{
						if (thatd.ContainsKey(s_ + s))
							thatd[s_ + s]++;
						else
							thatd[s_ + s] = 1;
					}
					s_ = s;
				}

				var thisl = new List<string>();
				s_ = null;
				foreach (var s in Regex.Split(thisText, @"", RegexOptions.Singleline))
				{
					if ((s ?? "") != "" && (s_ ?? "") != "")
					{
						thisl.Add(s_ + s);
					}
					s_ = s;
				}

				score = 0;
				foreach (var en in thisl)
				{
					if (thatd.ContainsKey(en))
					{
						score += 1F;
						thatd[en]--;
						if (thatd[en] == 0) thatd.Remove(en);
					}
				}

				Debug.Assert(thisl.Count > 0);

				score /= (float)thisl.Count;
			}

			return score;
			//return (that2.ToWikiText(parent).IndexOf(this.ToWikiText(parent)) != -1) ? 1F : 0F;
		}

		public Plaintext(PageElement parent, string content, HashSet<Type> ignoreClasses)
			: base(parent, ignoreClasses)
		{
			this.WikitextPre = content;
			this.WikitextPost = "";
			Debug.Assert(content.IndexOf("<") == -1 && content.IndexOf(">") == -1, "Plaintext:" + content);

			// contentをPlaintext系コンストラクターへ
			ignoreClasses.Add(this.GetType());
			var elementInstance = PageElements.Plain.Construct(parent, content, ignoreClasses);
			if (elementInstance != null)
				this.Children.Add(elementInstance);
			ignoreClasses.Remove(this.GetType());
		}
	}
}
